﻿using System;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Web;

namespace ELTE.TravelAgency.Models
{
    /// <summary>
    /// Felhasználókezelési szolgáltatás típusa.
    /// </summary>
    public class AccountService : IAccountService
    {
        private TravelAgencyEntities _entities;

        public AccountService()
        {
            _entities = new TravelAgencyEntities();

            // ha a felhasználónak van sütije, de még nincs bejelentkezve, bejelentkeztetjük
            if (HttpContext.Current.Request.Cookies["user"] != null && HttpContext.Current.Session["user"] == null)
            {
                HttpContext.Current.Session["user"] = HttpContext.Current.Request.Cookies["user"]["userName"]; // felvesszük a felhasználó nevét a munkamenetbe
                HttpContext.Current.Session.Timeout = 15; // max. 15 percig él a munkamenet

                UserCount++; // növeljük a felhasználószámot
            }
        }
        
        /// <summary>
        /// Felhasználószám lekérdezése.
        /// </summary>
        public Int32 UserCount
        {
            get
            {
                // a felhasználószámot globális állapotként tároljuk
                return HttpContext.Current.Application["userCount"] == null ? 0 : (Int32)HttpContext.Current.Application["userCount"];
            }
            set
            {
                HttpContext.Current.Application["userCount"] = value;
            }
        }

        /// <summary>
        /// Aktuálisan bejelentkezett felhasználó nevének lekérdezése.
        /// </summary>
        public String CurrentUserName
        {
            get { return HttpContext.Current.Session["user"] as String; }
        }

        /// <summary>
        /// Vendégadatok lekérdezése.
        /// </summary>
        /// <param name="userName">A felhasználónév.</param>
        public Guest GetGuest(String userName)
        {
            if (userName == null)
                return null;

            return _entities.Guest.FirstOrDefault(c => c.UserName == userName); // megkeressük a vendéget
        }

        /// <summary>
        /// Felhasználó bejelentkeztetése.
        /// </summary>
        /// <param name="user">A felhasználó nézetmodellje.</param>
        public Boolean Login(UserViewModel user)
        {
            if (user == null)
                return false;

            // ellenőrizzük az annotációkat
            if (!Validator.TryValidateObject(user, new ValidationContext(user, null, null), null))
                return false;

            Guest guest = _entities.Guest.FirstOrDefault(c => c.UserName == user.UserName); // megkeressük a felhasználót

            if (guest == null)
                return false;

            // ellenőrizzük a jelszót (ehhez a kapott jelszót hash-eljük)
            Byte[] passwordBytes = null;
            using (SHA512CryptoServiceProvider provider = new SHA512CryptoServiceProvider())
            {
                passwordBytes = provider.ComputeHash(Encoding.UTF8.GetBytes(user.UserPassword));
            }

            if (!passwordBytes.SequenceEqual(guest.UserPassword))
                return false;

            // ha sikeres volt az ellenőrzés
            HttpContext.Current.Session["user"] = user.UserName; // felvesszük a felhasználó nevét a munkamenetbe
            HttpContext.Current.Session.Timeout = 15; // max. 15 percig él a munkamenet

            if (user.RememberLogin) // ha meg kell jegyeznünk a felhasználót
            {
                HttpCookie cookie = new HttpCookie("user"); // akkor elküldjük azt sütiként
                cookie["userName"] = user.UserName;
                cookie.Expires = DateTime.Today.AddDays(365); // egy évig lesz érvényes a süti
                cookie.HttpOnly = true; // igyekszünk biztonságosság tenni a sütit
                cookie.Secure = true;
                HttpContext.Current.Response.Cookies.Add(cookie);
            }
            else // ha nem kell megjegyeznünk
            {
                // akkor kitöröljük a sütit (amennyiben volt)
                HttpCookie cookie = new HttpCookie("user");
                cookie.Expires = DateTime.MinValue; // azzal, hogy lejártnak minősítjük
                HttpContext.Current.Response.Cookies.Add(cookie);
            }

            // ha sikeres volt az ellenőrzés
            // módosítjuk a felhasználók számát
            UserCount++;

            return true;
        }
        
        /// <summary>
        /// Felhasználó kijelentkeztetése.
        /// </summary>
        public Boolean Logout()
        {
            if (HttpContext.Current.Session["user"] == null)
                return false;

            HttpContext.Current.Session["user"] = null;
            // töröljük a munkafolyamatból

            // módosítjuk a felhasználók számát
            UserCount--;

            return true;
        }

        /// <summary>
        /// Vendég regisztrációja.
        /// </summary>
        /// <param name="guest">A vendég nézetmodellje.</param>
        public Boolean Register(GuestRegistrationViewModel guest)
        {
            if (guest == null)
                return false;

            // ellenőrizzük az annotációkat
            if (!Validator.TryValidateObject(guest, new ValidationContext(guest, null, null), null))
                return false;

            if (_entities.Guest.Count(c => c.UserName == guest.UserName) != 0)
                return false;

            // kódoljuk a jelszót
            Byte[] passwordBytes = null;
            using (SHA512CryptoServiceProvider provider = new SHA512CryptoServiceProvider())
            {
                passwordBytes = provider.ComputeHash(Encoding.UTF8.GetBytes(guest.UserPassword));
            }

            // elmentjük a felhasználó adatait
            _entities.Guest.Add(new Guest
            {
                Name = guest.GuestName,
                Address = guest.GuestAddress,
                Email = guest.GuestEmail,
                PhoneNumber = guest.GuestPhoneNumber,
                UserName = guest.UserName,
                UserPassword = passwordBytes
            });

            try
            {
                _entities.SaveChanges();
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Vendég létrehozása (regisztráció nélkül).
        /// </summary>
        /// <param name="guest">A vendég nézetmodellje.</param>
        /// <param name="userName">A felhasznalónév.</param>
        public Boolean Create(GuestViewModel guest, out String userName)
        {
            userName = "user" + Guid.NewGuid(); // a felhasználónevet generáljuk

            if (guest == null)
                return false;

            // ellenőrizzük az annotációkat
            if (!Validator.TryValidateObject(guest, new ValidationContext(guest, null, null), null))
                return false;

            // elmentjük a felhasználó adatait
            _entities.Guest.Add(new Guest
            {
                Name = guest.GuestName,
                Address = guest.GuestAddress,
                Email = guest.GuestEmail,
                PhoneNumber = guest.GuestPhoneNumber,
                UserName = userName
            });
            
            try
            {
                _entities.SaveChanges();
            }
            catch
            {
                return false;
            }

            return true;
        }
    }
}